@@ -19,11 +19,9 @@ purposes only.
 """
 
 from barbican.api import middleware
-from barbican.common import config
 from barbican.common import utils
 
 LOG = utils.getLogger(__name__)
-CONF = config.CONF
 
 
 class SimpleFilter(middleware.Middleware):
