@@ -27,10 +27,14 @@ LOG = utils.getLogger(__name__)
 Attribute = collections.namedtuple("Attribute", ["type", "value"])
 CKAttributes = collections.namedtuple("CKAttributes", ["template", "cffivals"])
 CKMechanism = collections.namedtuple("CKMechanism", ["mech", "cffivals"])
+Token = collections.namedtuple("Token", ["slot_id", "label", "serial_number"])
 
 CKR_OK = 0
+CKR_CRYPTOKI_ALREADY_INITIALIZED = 0x00000191
+CK_TRUE = 1
 CKF_RW_SESSION = (1 << 1)
 CKF_SERIAL_SESSION = (1 << 2)
+CKF_OS_LOCKING_OK = 0x02
 CKU_SO = 0
 CKU_USER = 1
 
@@ -142,7 +146,7 @@ CKM_AES_KEY_WRAP = 0x1090
 CKM_GENERIC_SECRET_KEY_GEN = 0x350
 VENDOR_SAFENET_CKM_AES_GCM = 0x8000011c
 
-# Thales Vendor-defined Mechanisms
+# nCipher Vendor-defined Mechanisms
 CKM_NC_SHA256_HMAC_KEY_GEN = 0xDE436997
 
 _ENCRYPTION_MECHANISMS = {
@@ -263,15 +267,21 @@ def build_ffi():
     ffi = cffi.FFI()
     ffi.cdef(textwrap.dedent("""
     typedef unsigned char CK_BYTE;
+    typedef CK_BYTE CK_CHAR;
+    typedef CK_BYTE CK_UTF8CHAR;
+    typedef CK_BYTE CK_BBOOL;
     typedef unsigned long CK_ULONG;
     typedef unsigned long CK_RV;
     typedef unsigned long CK_SESSION_HANDLE;
     typedef unsigned long CK_OBJECT_HANDLE;
     typedef unsigned long CK_SLOT_ID;
+    typedef CK_SLOT_ID * CK_SLOT_ID_PTR;
     typedef unsigned long CK_FLAGS;
     typedef unsigned long CK_STATE;
     typedef unsigned long CK_USER_TYPE;
     typedef unsigned char * CK_UTF8CHAR_PTR;
+    typedef void * CK_VOID_PTR;
+    typedef CK_VOID_PTR * CK_VOID_PTR_PTR;
     typedef ... *CK_NOTIFY;
 
     typedef unsigned long ck_attribute_type_t;
@@ -283,6 +293,20 @@ def build_ffi():
     typedef struct ck_attribute CK_ATTRIBUTE;
     typedef CK_ATTRIBUTE *CK_ATTRIBUTE_PTR;
 
+    typedef CK_RV (*CK_CREATEMUTEX)(CK_VOID_PTR_PTR);
+    typedef CK_RV (*CK_DESTROYMUTEX)(CK_VOID_PTR);
+    typedef CK_RV (*CK_LOCKMUTEX)(CK_VOID_PTR);
+    typedef CK_RV (*CK_UNLOCKMUTEX)(CK_VOID_PTR);
+
+    typedef struct CK_C_INITIALIZE_ARGS {
+        CK_CREATEMUTEX CreateMutex;
+        CK_DESTROYMUTEX DestroyMutex;
+        CK_LOCKMUTEX LockMutex;
+        CK_UNLOCKMUTEX UnlockMutex;
+        CK_FLAGS flags;
+        CK_VOID_PTR pReserved;
+    } CK_C_INITIALIZE_ARGS;
+
     typedef unsigned long ck_mechanism_type_t;
     struct ck_mechanism {
         ck_mechanism_type_t mechanism;
@@ -294,6 +318,44 @@ def build_ffi():
     typedef CK_BYTE *CK_BYTE_PTR;
     typedef CK_ULONG *CK_ULONG_PTR;
 
+    typedef struct CK_VERSION {
+        CK_BYTE major;
+        CK_BYTE minor;
+    } CK_VERSION;
+
+    typedef struct CK_SLOT_INFO {
+        CK_UTF8CHAR slotDescription[64];
+        CK_UTF8CHAR manufacturerID[32];
+        CK_FLAGS    flags;
+
+        CK_VERSION  hardwareVersion;
+        CK_VERSION  firmwareVersion;
+    } CK_SLOT_INFO;
+    typedef CK_SLOT_INFO * CK_SLOT_INFO_PTR;
+
+    typedef struct CK_TOKEN_INFO {
+        CK_UTF8CHAR label[32];
+        CK_UTF8CHAR manufacturerID[32];
+        CK_UTF8CHAR model[16];
+        CK_CHAR serialNumber[16];
+        CK_FLAGS flags;
+
+        CK_ULONG ulMaxSessionCount;
+        CK_ULONG ulSessionCount;
+        CK_ULONG ulMaxRwSessionCount;
+        CK_ULONG ulRwSessionCount;
+        CK_ULONG ulMaxPinLen;
+        CK_ULONG ulMinPinLen;
+        CK_ULONG ulTotalPublicMemory;
+        CK_ULONG ulFreePublicMemory;
+        CK_ULONG ulTotalPrivateMemory;
+        CK_ULONG ulFreePrivateMemory;
+        CK_VERSION hardwareVersion;
+        CK_VERSION firmwareVersion;
+        CK_CHAR utcTime[16];
+    } CK_TOKEN_INFO;
+    typedef CK_TOKEN_INFO * CK_TOKEN_INFO_PTR;
+
     typedef struct ck_session_info {
         CK_SLOT_ID slot_id;
         CK_STATE state;
@@ -321,6 +383,9 @@ def build_ffi():
     CK_RV C_GetSessionInfo(CK_SESSION_HANDLE, CK_SESSION_INFO_PTR);
     CK_RV C_Login(CK_SESSION_HANDLE, CK_USER_TYPE, CK_UTF8CHAR_PTR,
                   CK_ULONG);
+    CK_RV C_GetSlotList(CK_BBOOL, CK_SLOT_ID_PTR, CK_ULONG_PTR);
+    CK_RV C_GetSlotInfo(CK_SLOT_ID, CK_SLOT_INFO_PTR);
+    CK_RV C_GetTokenInfo(CK_SLOT_ID, CK_TOKEN_INFO_PTR);
     CK_RV C_GetAttributeValue(CK_SESSION_HANDLE, CK_OBJECT_HANDLE,
                               CK_ATTRIBUTE *, CK_ULONG);
     CK_RV C_SetAttributeValue(CK_SESSION_HANDLE, CK_OBJECT_HANDLE,
@@ -365,7 +430,10 @@ class PKCS11(object):
                  ffi=None, algorithm=None,
                  seed_random_buffer=None,
                  generate_iv=None, always_set_cka_sensitive=None,
-                 hmac_keywrap_mechanism='CKM_SHA256_HMAC'):
+                 hmac_keywrap_mechanism='CKM_SHA256_HMAC',
+                 token_serial_number=None,
+                 token_labels=None,
+                 os_locking_ok=False):
         if algorithm:
             LOG.warning("WARNING: Using deprecated 'algorithm' argument.")
             encryption_mechanism = encryption_mechanism or algorithm
@@ -383,13 +451,23 @@ class PKCS11(object):
 
         self.ffi = ffi or build_ffi()
         self.lib = self.ffi.dlopen(library_path)
-        rv = self.lib.C_Initialize(self.ffi.NULL)
+
+        if os_locking_ok:
+            init_arg_pt = self.ffi.new("CK_C_INITIALIZE_ARGS *")
+            init_arg_pt.flags = CKF_OS_LOCKING_OK
+        else:
+            init_arg_pt = self.ffi.NULL
+
+        rv = self.lib.C_Initialize(init_arg_pt)
         self._check_error(rv)
 
         # Session options
         self.login_passphrase = _to_bytes(login_passphrase)
         self.rw_session = rw_session
-        self.slot_id = slot_id
+        self.slot_id = self._get_slot_id(
+            token_serial_number,
+            token_labels,
+            slot_id)
 
         # Algorithm options
         self.algorithm = CKM_NAMES[encryption_mechanism]
@@ -406,6 +484,70 @@ class PKCS11(object):
             self._seed_random(session, seed_random_buffer)
         self._rng_self_test(session)
         self.return_session(session)
+        LOG.debug("Connected to PCKS#11 Token in Slot %s", self.slot_id)
+
+    def _get_slot_id(self, token_serial_number, token_labels, slot_id):
+        # First find out how many slots with tokens are available
+        slots_ptr = self.ffi.new("CK_ULONG_PTR")
+        rv = self.lib.C_GetSlotList(CK_TRUE, self.ffi.NULL, slots_ptr)
+        self._check_error(rv)
+
+        # Next get the Slot IDs for each of the available slots
+        slot_ids_ptr = self.ffi.new("CK_SLOT_ID[{}]".format(slots_ptr[0]))
+        rv = self.lib.C_GetSlotList(CK_TRUE, slot_ids_ptr, slots_ptr)
+        self._check_error(rv)
+
+        # Gather details from each token
+        tokens = list()
+        for id in slot_ids_ptr:
+            token_info_ptr = self.ffi.new("CK_TOKEN_INFO_PTR")
+            rv = self.lib.C_GetTokenInfo(id, token_info_ptr)
+            self._check_error(rv)
+            token = Token(
+                id,
+                self.ffi.string(token_info_ptr.label).decode("UTF-8").strip(),
+                self.ffi.string(
+                    token_info_ptr.serialNumber
+                ).decode("UTF-8").strip()
+            )
+            LOG.debug("Slot %s: label: %s sn: %s",
+                      token.slot_id,
+                      token.label,
+                      token.serial_number)
+            tokens.append(token)
+
+        # Matching serial number gets highest priority
+        if token_serial_number:
+            for token in tokens:
+                if token.serial_number == token_serial_number:
+                    LOG.debug("Found token sn: %s in slot %s",
+                              token.serial_number,
+                              token.slot_id)
+                    if token_labels:
+                        LOG.warning(
+                            "Ignoring token_labels: %s from barbican.conf",
+                            token_labels
+                        )
+                    if slot_id:
+                        LOG.warning("Ignoring slot_id: %s from barbican.conf",
+                                    slot_id)
+                    return token.slot_id
+            raise ValueError("Token Serial Number not found.")
+
+        # Label match is next, raises an error if there's not exactly one match
+        if token_labels:
+            for token in tokens:
+                if token.label in token_labels:
+                    LOG.debug("Found token label: %s in slot %s", token.label,
+                              token.slot_id)
+                    if slot_id:
+                        LOG.warning("Ignoring slot_id: %s from barbican.conf",
+                                    slot_id)
+                    return token.slot_id
+            raise ValueError("Token Labels not found.")
+
+        # If we got this far, slot_id was the only param given, so we return it
+        return slot_id
 
     def get_session(self):
         session = self._open_session(self.slot_id)
@@ -718,7 +860,7 @@ class PKCS11(object):
         self._check_error(rv)
 
     def _check_error(self, value):
-        if value != CKR_OK:
+        if value != CKR_OK and value != CKR_CRYPTOKI_ALREADY_INITIALIZED:
             code = ERROR_CODES.get(value, 'CKR_????')
             hex_code = "{hex} {code}".format(hex=hex(value), code=code)
 
