@@ -33,12 +33,12 @@ from barbican.tests import utils
 
 def get_barbican_env(external_project_id):
     class NoopPolicyEnforcer(object):
-        def enforce(self, *args, **kwargs):
+        def authorize(self, *args, **kwargs):
             return
 
     kwargs = {'roles': None,
-              'user': None,
-              'project': external_project_id,
+              'user_id': None,
+              'project_id': external_project_id,
               'is_admin': True,
               'policy_enforcer': NoopPolicyEnforcer()}
     barbican_env = {'barbican.context':
